# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_vertical_moment_01():

    score = Score([])
    staff_1 = Staff([Tuplet((4, 3), "d''8 c''8 b'8")])
    score.append(staff_1)
    staff_group = StaffGroup(context_name='PianoStaff')
    staff_group.extend([Staff("a'4 g'4"), Staff("f'8 e'8 d'8 c'8")])
    clef = Clef('bass')
    attach(clef, staff_group[1])
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    moment_generator = iterate(score).by_vertical_moment(reverse=True)
    moments = list(moment_generator)

    r'''
    (Note(b', 8), Note(g', 4), Note(c', 8))
    (Note(b', 8), Note(g', 4), Note(d', 8))
    (Note(c'', 8), Note(g', 4), Note(d', 8))
    (Note(c'', 8), Note(a', 4), Note(e', 8))
    (Note(d'', 8), Note(a', 4), Note(e', 8))
    (Note(d'', 8), Note(a', 4), Note(f', 8))
    '''

    tuplet = list(iterate(score[0][0]).by_leaf())
    treble = list(iterate(staff_group[0]).by_leaf())
    bass = list(iterate(staff_group[1]).by_leaf())

    assert moments[0].leaves == (tuplet[2], treble[1], bass[3])
    assert moments[1].leaves == (tuplet[2], treble[1], bass[2])
    assert moments[2].leaves == (tuplet[1], treble[1], bass[2])
    assert moments[3].leaves == (tuplet[1], treble[0], bass[1])
    assert moments[4].leaves == (tuplet[0], treble[0], bass[1])
    assert moments[5].leaves == (tuplet[0], treble[0], bass[0])


def test_agenttools_IterationAgent_by_vertical_moment_02():

    score = Score([])
    staff_1 = Staff([Tuplet((4, 3), "d''8 c''8 b'8")])
    score.append(staff_1)
    staff_group = StaffGroup(context_name='PianoStaff')
    staff_group.extend([Staff("a'4 g'4"), Staff("f'8 e'8 d'8 c'8")])
    clef = Clef('bass')
    attach(clef, staff_group[1])
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    # see above for formatted score #

    moment_generator = iterate(staff_group).by_vertical_moment(reverse=True)
    moments = list(moment_generator)

    r'''
    (Note(g', 4), Note(c', 8))
    (Note(g', 4), Note(d', 8))
    (Note(a', 4), Note(e', 8))
    (Note(a', 4), Note(f', 8))
    '''

    treble = list(iterate(staff_group[0]).by_leaf())
    bass = list(iterate(staff_group[1]).by_leaf())

    assert moments[0].leaves == (treble[1], bass[3])
    assert moments[1].leaves == (treble[1], bass[2])
    assert moments[2].leaves == (treble[0], bass[1])
    assert moments[3].leaves == (treble[0], bass[0])


def test_agenttools_IterationAgent_by_vertical_moment_03():

    score = Score([])
    staff_1 = Staff([Tuplet((4, 3), "d''8 c''8 b'8")])
    score.append(staff_1)
    staff_group = StaffGroup(context_name='PianoStaff')
    staff_group.extend([Staff("a'4 g'4"), Staff("f'8 e'8 d'8 c'8")])
    clef = Clef('bass')
    attach(clef, staff_group[1])
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    moment_generator = iterate(score).by_vertical_moment()
    moments = list(moment_generator)

    r'''
    (Note(d'', 8), Note(a', 4), Note(f', 8))
    (Note(d'', 8), Note(a', 4), Note(e', 8))
    (Note(c'', 8), Note(a', 4), Note(e', 8))
    (Note(c'', 8), Note(g', 4), Note(d', 8))
    (Note(b', 8), Note(g', 4), Note(d', 8))
    (Note(b', 8), Note(g', 4), Note(c', 8))
    '''

    tuplet = list(iterate(score[0][0]).by_leaf())
    treble = list(iterate(staff_group[0]).by_leaf())
    bass = list(iterate(staff_group[1]).by_leaf())

    assert set(moments[0].leaves) == set((tuplet[0], treble[0], bass[0]))
    assert set(moments[1].leaves) == set((tuplet[0], treble[0], bass[1]))
    assert set(moments[2].leaves) == set((tuplet[1], treble[0], bass[1]))
    assert set(moments[3].leaves) == set((tuplet[1], treble[1], bass[2]))
    assert set(moments[4].leaves) == set((tuplet[2], treble[1], bass[2]))
    assert set(moments[5].leaves) == set((tuplet[2], treble[1], bass[3]))


def test_agenttools_IterationAgent_by_vertical_moment_04():

    score = Score([])
    staff_1 = Staff([Tuplet((4, 3), "d''8 c''8 b'8")])
    score.append(staff_1)
    staff_group = StaffGroup(context_name='PianoStaff')
    staff_group.extend([Staff("a'4 g'4"), Staff("f'8 e'8 d'8 c'8")])
    clef = Clef('bass')
    attach(clef, staff_group[1])
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    moment_generator = iterate(staff_group).by_vertical_moment()
    moments = list(moment_generator)

    r'''
    (Note(a', 4), Note(f', 8))
    (Note(a', 4), Note(e', 8))
    (Note(g', 4), Note(d', 8))
    (Note(g', 4), Note(c', 8))
    '''

    treble = list(iterate(staff_group[0]).by_leaf())
    bass = list(iterate(staff_group[1]).by_leaf())

    assert set(moments[0].leaves) == set((treble[0], bass[0]))
    assert set(moments[1].leaves) == set((treble[0], bass[1]))
    assert set(moments[2].leaves) == set((treble[1], bass[2]))
    assert set(moments[3].leaves) == set((treble[1], bass[3]))


# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList


class PerformerList(TypedList):
    r'''Performer list.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PUBLIC METHODS ###

    def get_instrument(self, instrument_name):
        r'''Gets first instrument in performer list with `instrument_name`.

        ..  container:: example

            ::
            
                >>> flutist = instrumenttools.Performer(name='flutist')
                >>> flutist.instruments.append(instrumenttools.Flute())
                >>> flutist.instruments.append(instrumenttools.Piccolo())
                >>> performers = instrumenttools.PerformerList(
                ...     [flutist],
                ...     )
                >>> performers.get_instrument('piccolo')
                Piccolo()

        Returns instrument or none.
        '''
        for performer in self:
            instrument = performer.get_instrument(instrument_name)
            if instrument is not None:
                return instrument



# -*- coding: utf-8 -*-
import pickle
from abjad.tools import datastructuretools


def test_datastructuretools_TreeContainer_pickle_01():

    container = datastructuretools.TreeContainer()
    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()
    container.extend((leaf_a, leaf_b, leaf_c))

    pickled = pickle.loads(pickle.dumps(container))

    assert format(container) == format(pickled)



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Voice___copy___01():
    r'''Voices copy name, engraver removals, engraver consists,
    grob overrides and context settings. Voices do not copy musical
    content.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_1.name = 'SopranoVoice'
    voice_1.remove_commands.append('Forbid_line_break_engraver')
    voice_1.consists_commands.append('Time_signature_engraver')
    override(voice_1).note_head.color = 'red'
    set_(voice_1).tuplet_full_length = True
    voice_2 = copy.copy(voice_1)

    assert format(voice_2) == stringtools.normalize(
        r'''
        \context Voice = "SopranoVoice" \with {
            \remove Forbid_line_break_engraver
            \consists Time_signature_engraver
            \override NoteHead.color = #red
            tupletFullLength = ##t
        } {
        }
        '''
        )


def test_scoretools_Voice___copy___02():
    r'''Voice copies semanticity flag.
    '''

    voice_1 = Voice("s8 s8 s8 s8")
    voice_1.name = 'SkipVoice'
    voice_1.is_nonsemantic = True

    assert format(voice_1) == stringtools.normalize(
        r'''
        \context Voice = "SkipVoice" {
            s8
            s8
            s8
            s8
        }
        '''
        )

    voice_2 = copy.copy(voice_1)

    assert format(voice_2) == stringtools.normalize(
        r'''
        \context Voice = "SkipVoice" {
        }
        '''
        )

    assert voice_2.is_nonsemantic



from abjad import *


def test_scoretools_Voice___delitem___01():
    r'''Delete container from voice.
    '''

    voice = Voice(r'''
        c'8 [ \glissando
        {
            d'8 \glissando
            e'8 \glissando
        }
        f'8 ]
        ''')

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            {
                d'8 \glissando
                e'8 \glissando
            }
            f'8 ]
        }
        '''
        )

    container = voice[1]
    del(voice[1:2])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [ \glissando
            f'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert inspect_(container).is_well_formed()



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import durationtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import detach


def scale_measure_denominator_and_adjust_measure_contents(measure, factor):
    r'''Scales power-of-two `measure` to non-power-of-two measure
    with new denominator `factor`:

    ..  container:: example

        ::

            >>> measure = Measure((2, 8), "c'8 d'8")
            >>> measure.implicit_scaling = True
            >>> beam = spannertools.Beam()
            >>> attach(beam, measure[:])
            >>> show(measure) # doctest: +SKIP

        ..  doctest::

            >>> print(format(measure))
            {
                \time 2/8
                c'8 [
                d'8 ]
            }

        ::

            >>> scoretools.scale_measure_denominator_and_adjust_measure_contents(
            ...     measure, 3)
            Measure((3, 12), "c'8. d'8.", implicit_scaling=True)
            >>> show(measure) # doctest: +SKIP

        ..  doctest::

            >>> print(format(measure))
            {
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8. [
                    d'8. ]
                }
            }

    Treats new denominator `factor` like clever form of ``1``:
    ``3/3`` or ``5/5`` or ``7/7``, etc.

    Preserves `measure` duration.

    Derives new `measure` multiplier.

    Scales `measure` contents.

    Picks best new time signature.
    '''
    from abjad.tools import scoretools

    # save old time signature duration
    old_time_signature_duration = measure.time_signature.duration

    # find new time signature
    new_time_signature = \
        measure._duration_and_possible_denominators_to_time_signature(
        old_time_signature_duration,
        factor=factor,
        )

    # scale contents of measures in argument
    multiplier = new_time_signature.implied_prolation.reciprocal
    measure._scale(multiplier)

    # assign new time signature
    detach(indicatortools.TimeSignature, measure)
    attach(new_time_signature, measure)
    if new_time_signature.has_non_power_of_two_denominator:
        measure.implicit_scaling = True

    # return measure
    return measure


# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import select
from abjad.tools.selectiontools.Selection import Selection


class Descendants(Selection):
    r'''A selection of components that descend from a component.

    ::

        >>> score = Score()
        >>> score.append(Staff(r"""\new Voice = "Treble Voice" { c'4 }""",
        ...     name='Treble Staff'))
        >>> score.append(Staff(r"""\new Voice = "Bass Voice" { b,4 }""",
        ...     name='Bass Staff'))

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \context Staff = "Treble Staff" {
                \context Voice = "Treble Voice" {
                    c'4
                }
            }
            \context Staff = "Bass Staff" {
                \context Voice = "Bass Voice" {
                    b,4
                }
            }
        >>

    ::

        >>> for x in selectiontools.Descendants(score): x
        ...
        <Score<<2>>>
        <Staff-"Treble Staff"{1}>
        Voice("c'4")
        Note("c'4")
        <Staff-"Bass Staff"{1}>
        Voice('b,4')
        Note('b,4')

    ::

        >>> for x in selectiontools.Descendants(score['Bass Voice']): x
        ...
        Voice('b,4')
        Note('b,4')

    Descendants is treated as the selection of the component's
    improper descendants.

    Returns descendants.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_component',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        component=None,
        cross_offset=None,
        include_self=True,
        ):
        from abjad.tools import scoretools
        assert isinstance(component, (scoretools.Component, type(None)))
        if component is None:
            music = ()
        else:
            music = list(select(component).by_class())
            if not include_self:
                music.remove(component)
        result = []
        if cross_offset is None:
            result = music
        else:
            for x in music:
                append_x = True
                if not (x._get_timespan().start_offset < cross_offset and
                    cross_offset < x._get_timespan().stop_offset):
                    append_x = False
                if append_x:
                    result.append(x)
        Selection.__init__(self, result)
        self._component = component

    ### PUBLIC PROPERTIES ###

    @property
    def component(self):
        r'''The component from which the selection was derived.
        '''
        return self._component



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Articulation___setattr___01():
    r'''Slots constrain articulation attributes.
    '''

    articulation = Articulation('staccato')

    assert pytest.raises(AttributeError, "articulation.foo = 'bar'")
